@@ -157,7 +157,7 @@ void VJ_subproc(enum jail_subproc_e);
 int VJ_make_workdir(const char *);
 int VJ_make_subdir(const char *, const char *, struct vsb *);
 void VJ_fix_fd(int, enum jail_fixfd_e);
-void VJ_unlink(const char *);
+void VJ_unlink(const char *, int);
 void VJ_rmdir(const char *);
 
 extern const struct jail_tech jail_tech_unix;
@@ -196,7 +196,7 @@ extern struct params mgt_param;
 /* mgt_shmem.c */
 void mgt_SHM_Init(void);
 void mgt_SHM_static_alloc(const void *, ssize_t size,
-    const char *class, const char *ident);
+    const char *category, const char *ident);
 void mgt_SHM_Create(void);
 void mgt_SHM_Destroy(int keep);
 
